# -*- coding: utf-8 -*-
###############################################################################
#
#    Copyright (C) 2001-2014 Micronaet SRL (<http://www.micronaet.it>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###############################################################################
import os
import sys
import netsvc
import logging
from osv import osv, fields
from datetime import datetime, timedelta
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT, DEFAULT_SERVER_DATETIME_FORMAT, DATETIME_FORMATS_MAP, float_compare
import openerp.addons.decimal_precision as dp
from openerp.tools.translate import _


_logger = logging.getLogger(__name__)

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:


# -----------------------------------------------------------------------------
#                                    CLAIMS:
# -----------------------------------------------------------------------------
class quality_origin(osv.osv):
    ''' Simple anagrafic for origin
    '''
    _name = 'quality.origin'
    _description = 'Origin'

    _columns = {
        'name':fields.char('Description', size=64, required=True, readonly=False),
        'note': fields.text('Note'),
        'type':fields.selection([
            ('claim','Claim'),
            ('action','Action'),
        ], 'Type', select=True, readonly=False),
    }

class quality_claim_cause(osv.osv):
    ''' Simple anagrafic for cause
    '''
    _name = 'quality.claim.cause'
    _description = 'Claim cause'

    _columns = {
        'name':fields.char('Description', size=64, required=True, readonly=False),
        'note': fields.text('Note'),
    }

class quality_claim_gravity(osv.osv):
    ''' Simple anagrafic for gravity
    '''
    _name = 'quality.claim.gravity'
    _description = 'Claim gravity'

    _columns = {
        'name':fields.char('Description', size=64, required=True, readonly=False),
        'note': fields.text('Note'),
    }

class quality_claim(osv.osv):
    _name = 'quality.claim'
    _description = 'Claim'
    _rec_name = 'ref'
    
    # -------
    # Button:
    # -------
    def create_action(self, cr, uid, ids, context=None):
        ''' Create a Action and link to this Claim
        '''
        claim_proxy = self.browse(cr, uid, ids, context=context)[0]
        action_pool = self.pool.get('quality.action')
        action_id = action_pool.create(cr, uid, {
            'ref': "ACAP%s" % (claim_proxy.ref[3:],),
            'name': claim_proxy.name,
            'date': datetime.now().strftime(DEFAULT_SERVER_DATE_FORMAT),
            'claim_id': ids[0],
            'origin_id': 4, # TODO change
            'type': 'corrective',
            #'': 
        }, context=context)
        self.write(cr, uid, ids, {'action_id': action_id, }, context=context)
        return True# { # New Action created:

    def create_sampling(self, cr, uid, ids, context=None):
        ''' Create a Sampling form and link to this Claim
        '''
        return True
        
    _columns = {
        'name':fields.char('Description', size=64, required=True, readonly=False),
        'ref': fields.char('Ref', size=12, required=True, readonly=False),
        'date': fields.date('Date'),
        'closed_date': fields.date('Closed date'),

        'subject': fields.text('Subject'),
        'analysis': fields.text('Returned analysis'),
        'responsability': fields.text('Responsability'),
        'solution': fields.text('Concorded solution'),
        
        'partner_id':fields.many2one('res.partner', 'Customer', required=True),
        'partner_ref':fields.char('Customer ref', size=64, required=False, readonly=False),
        'receive_user_id':fields.many2one('res.users', 'Receive user', required=True),
        'insert_user_id':fields.many2one('res.users', 'Insert user', required=True),

        'request_return': fields.boolean('Product return request', required=False),
        'RAQ_saw': fields.boolean('RAQ saw', required=False),
        'RTR_request': fields.boolean('RTR request', required=False),
        
        'NC':fields.boolean('NC required', required=False),
        'NC_ref': fields.char('NC ref', size=15, required=False, readonly=False),
        'SFA_saw': fields.boolean('SFA saw', required=False),
        'RAQ_confirm': fields.boolean('RAQ confirm', required=False), # TODO serve?

        'origin_id': fields.many2one('quality.origin', 'Origin', required=False),
        'cause_id': fields.many2one('quality.claim.cause', 'Cause', required=False),
        'gravity_id': fields.many2one('quality.claim.gravity', 'Gravity', required=False),

        #'one': fields.integer('Total'),
        
        'state':fields.selection([
            ('draft', 'Draft'),
            ('opened', 'Opened'),
            ('closed', 'Closed'),
            ('cancel', 'Cancel'),
        ],'State', select=True, readonly=True),
    }

    _defaults = {
        'date': lambda *x: datetime.now().strftime(DEFAULT_SERVER_DATE_FORMAT),
        'receive_user_id': lambda s, cr, uid, ctx: uid,
        'insert_user_id': lambda s, cr, uid, ctx: uid,
        'request_return': lambda *x: False,
        'RAQ_saw': lambda *x: False,
        'RTR_request': lambda *x: False,
        'NC': lambda *x: False,
        'SFA_saw': lambda *x: False,
        'state': lambda *a: 'draft',
        #'one': lambda *x: 1,
    }

    #=============================#
    # Workflow Activity Function: #
    #=============================#

    def claim_draft(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {
                    'state': 'draft',
                    }, context=context)

    def claim_opened(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {
                    'state': 'opened',
                    }, context=context)

    def claim_closed(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {
                    'state': 'closed',
                    }, context=context)

    def claim_cancel(self, cr, uid, ids, context=None):
        return self.write(cr, uid, ids, {
                    'state': 'cancel',
                    }, context=context)

class quality_claim_product(osv.osv):
    ''' List of product / lot claimed 
    '''
    _name = 'quality.claim.product'
    _description = 'Claim product'
    _rec_name = 'lot_id'

    # On change
    def onchange_lot_id(self, cr, uid, ids, lot_id, context=None):
        ''' Changing lot > find product
        '''
        res = {}
        if lot_id:
            lot_pool = self.pool.get('stock.production.lot')
            lot_proxy = lot_pool.browse(cr, uid, lot_id, context=context)
            res['value'] = {}
            try:
                print lot_proxy.product_id.id
                res['product_id'] = lot_proxy.product_id.id
            except:
                pass
        return res

    _columns = {
        'return_date': fields.date('Return date'),
        'return_qty': fields.float('Return q.', digits=(16, 3)),
        'lot_id':fields.many2one('stock.production.lot', 'Lot', required=True),
        #'supplier_id': fields.related('lot_id','supplier_id', type='many2one', relation='res.partner', string='Supplier'),
        'product_id': fields.related('lot_id', 'product_id', type='many2one', relation='product.product', string='Product'),
        'claim_id':fields.many2one('quality.claim', 'Claim', required=False),
    }

# -----------------------------------------------------------------------------
#                                    ACCEPTATION
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
#                                     SAMPLING          
# -----------------------------------------------------------------------------

# -----------------------------------------------------------------------------
#                            ACTIONS (CORREPTIVE, PREVENTIVE)
# -----------------------------------------------------------------------------
class quality_action(osv.osv):
    ''' Correptive / preventive action
    '''
    _name = 'quality.action'
    _description = 'Action'    
    _rec_name = 'ref'
    
    _columns = {
        'name': fields.char('Subject', size=100, required=False, readonly=False),
        'ref': fields.char('Ref', size=12, required=True, readonly=False),
        'date': fields.date('Date'),
        'origin_id': fields.many2one('quality.origin', 'Origin', required=False),
        'note': fields.text('Note'),
        'proposed_subject': fields.text('Subject proposing'),
        
        'esit_date': fields.date('Esit date'),
        'closed_date': fields.date('Closed date'),
        'esit_note': fields.text('Esit note'),

        'claim_id': fields.many2one('quality.claim', 'Claim', required=False),
        'type': fields.selection([
            ('corrective', 'Corrective'),
            ('preventive', 'Preventive'),            
        ], 'Type', select=True),
    }

class quality_action_intervent(osv.osv):
    ''' Correptive / preventive action intervent
    '''
    _name = 'quality.action.intervent'
    _description = 'Action intervent'

    _columns = {
        'name': fields.text('Description', size=100, required=False, readonly=False),
        'manager_id': fields.many2one('res.users', 'Manager', required=False),
        'deadline': fields.date('Deadline'),
        'action_id': fields.many2one('quality.action', 'Action', required=False),
        #'action_state': fields.related('action_id','state', type='selction', string='State'),
    }

class quality_action(osv.osv):
    ''' Update extra field for action
    '''
    _name = 'quality.action'
    _inherit = 'quality.action'

    _columns = {
        'child_id': fields.many2one('quality.action', 'Child', required=False),
        'parent_id': fields.many2one('quality.action', 'Parent', required=False),
        'intervent_ids': fields.one2many('quality.action.intervent', 'action_id', 'Intervent', required=False),
        'claim_id': fields.many2one('quality.claim', 'Claim', required=False, ondelete='set null'),
    }    

# -----------------------------------------------------------------------------
#                                   NOT CONFORMED
# -----------------------------------------------------------------------------


# -----------------------------------------------------------------------------
#                                        RELATIONS:
# -----------------------------------------------------------------------------
class quality_claim(osv.osv):
    ''' Assign *2many fields to claims
    '''
    _name = 'quality.claim'
    _inherit = 'quality.claim'

    _columns = {
        'product_ids':fields.one2many('quality.claim.product', 'claim_id', 'Product', required=False),
        'action_id': fields.many2one('quality.action', 'Action', required=False, ondelete='set null'),
    }

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
